"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _child_process = _interopRequireDefault(require("child_process"));

var _fs = _interopRequireDefault(require("fs"));

var _util = require("util");

var _execa = _interopRequireDefault(require("execa"));

var _dotProp = _interopRequireDefault(require("dot-prop"));

var _ink = require("ink");

var _termSize = _interopRequireDefault(require("term-size"));

var _inkTextInput = _interopRequireDefault(require("ink-text-input"));

var _inkSelectInput = _interopRequireDefault(require("ink-select-input"));

var _inkSpinner = _interopRequireDefault(require("ink-spinner"));

var _algolia = require("./algolia");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

const canAccessFile = (0, _util.promisify)(_fs.default.access);
const exec = (0, _util.promisify)(_child_process.default.exec); // Helpers -------------------------------------------------------------------
// Terminal

const maxCellSize = () => (0, _termSize.default)().columns / 4; // Yarn


const isYarnInstalled =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* () {
    try {
      yield exec(`yarnpkg --version`, {
        stdio: `ignore`
      });
      return true;
    } catch (err) {
      return false;
    }
  });

  return function isYarnInstalled() {
    return _ref.apply(this, arguments);
  };
}();

const shouldUseYarn =
/*#__PURE__*/
function () {
  var _ref2 = _asyncToGenerator(function* () {
    try {
      yield canAccessFile('package-lock.json');
      return false;
    } catch (err) {
      return isYarnInstalled();
    }
  });

  return function shouldUseYarn() {
    return _ref2.apply(this, arguments);
  };
}(); // Additional


const notEmpty = x => x.length !== 0;

const isEmpty = x => x.length === 0;

const getCellPadding = (pkgs, pkg) => attr => {
  const cells = pkgs.map(_pkg => _dotProp.default.get(_pkg, attr));
  const cellWidth = Math.max(...cells.map(cell => cell ? cell.length : 0));
  const cellValueWidth = _dotProp.default.get(pkg, attr) === null ? 0 : _dotProp.default.get(pkg, attr).length;
  const width = cellWidth - cellValueWidth;
  return ` `.repeat(width);
}; // Progress


const PROGRESS_NOT_LOADED = 0;
const PROGRESS_LOADING = 1;
const PROGRESS_LOADED = 2;
const PROGRESS_ERROR = 3; // Emma ----------------------------------------------------------------------
// Package

const PackageAttribute = (_ref3) => {
  let {
    pkg,
    attr
  } = _ref3,
      props = _objectWithoutProperties(_ref3, ["pkg", "attr"]);

  return (0, _ink.h)(_ink.Text, props, `${_dotProp.default.get(pkg, attr)} ${pkg._cell(attr)}`.slice(0, maxCellSize()));
};

const Package = pkg => (0, _ink.h)(_ink.Text, null, (0, _ink.h)(PackageAttribute, {
  pkg: pkg,
  attr: "humanDownloadsLast30Days"
}), (0, _ink.h)(PackageAttribute, {
  pkg: pkg,
  attr: "name",
  blueBright: true,
  bold: true
}), (0, _ink.h)(PackageAttribute, {
  pkg: pkg,
  attr: "owner.name",
  cyan: true
}), (0, _ink.h)(PackageAttribute, {
  pkg: pkg,
  attr: "description",
  bold: true
})); // Search


const Search = ({
  value,
  onChange,
  onSubmit
}) => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  bold: true,
  white: true
}, `Search packages 📦  : `), (0, _ink.h)(_inkTextInput.default, {
  value: value,
  onChange: onChange,
  onSubmit: onSubmit,
  placeholder: "..."
})); // Overview


const SelectedPackage = ({
  pkg
}) => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  magenta: true
}, ` ›`), (0, _ink.h)(_ink.Text, {
  bold: true,
  white: true
}, ` ${pkg.name} `), (0, _ink.h)(_ink.Text, {
  grey: true
}, ` ${pkg.version} `));

const SelectedPackages = ({
  selectedPackages
}) => (0, _ink.h)("div", null, (0, _ink.h)("div", null), (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  bold: true,
  white: true
}, "Picked: ")), selectedPackages.map(pkg => (0, _ink.h)(SelectedPackage, {
  key: pkg.name,
  pkg: pkg
}))); // Restults


const SearchResults = ({
  foundPackages,
  onToggle,
  loading
}) => {
  return (0, _ink.h)("span", null, (0, _ink.h)(_inkSelectInput.default, {
    items: foundPackages,
    itemComponent: Package,
    onSelect: onToggle
  }), isEmpty(foundPackages) && (0, _ink.h)(NotFoundInfo, null), (0, _ink.h)(AlgoliaInfo, null), loading === PROGRESS_LOADING && (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
    bold: true
  }, (0, _ink.h)(_inkSpinner.default, {
    red: true
  }), " Fetching")));
}; // Info


const SearchInfo = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  grey: true
}, "Try typing in to search the database."));

const InstallInfo = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  grey: true
}, "Press enter to install all of your packages."));

const NotFoundInfo = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  grey: true
}, `We couldn't find any package that would match your input...`));

const ErrorInfo = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  red: true
}, "Check your internet connection."));

const AlgoliaInfo = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, null, "Search powered by"), (0, _ink.h)(_ink.Text, {
  blue: true
}, " Algolia"), (0, _ink.h)(_ink.Text, null, ".")); // Emma


class Emma extends _ink.Component {
  constructor(props) {
    super(props);
    this.state = {
      query: '',
      foundPackages: [],
      selectedPackages: [],
      loading: PROGRESS_NOT_LOADED
    };
    this.handleQueryChange = this.handleQueryChange.bind(this);
    this.handleInstall = this.handleInstall.bind(this);
    this.handleTogglePackage = this.handleTogglePackage.bind(this);
  }

  render() {
    const {
      query,
      foundPackages,
      selectedPackages,
      loading
    } = this.state;
    return (0, _ink.h)("div", null, (0, _ink.h)(Search, {
      value: query,
      onChange: this.handleQueryChange,
      onSubmit: this.handleInstall,
      loading: loading
    }), loading === PROGRESS_NOT_LOADED && (0, _ink.h)(SearchInfo, null), isEmpty(query) && (0, _ink.h)(InstallInfo, null), loading === PROGRESS_ERROR && (0, _ink.h)(ErrorInfo, null), notEmpty(query) && (0, _ink.h)(SearchResults, {
      foundPackages: foundPackages,
      onToggle: this.handleTogglePackage,
      loading: loading
    }), notEmpty(selectedPackages) && (0, _ink.h)(SelectedPackages, {
      selectedPackages: selectedPackages
    }));
  }

  handleQueryChange(query) {
    var _this = this;

    return _asyncToGenerator(function* () {
      _this.setState({
        query,
        loading: PROGRESS_LOADING
      });

      try {
        const res = yield _this.fetchPackages(query);

        if (_this.state.query === query) {
          _this.setState({
            foundPackages: res,
            loading: PROGRESS_LOADED
          });
        }
      } catch (err) {
        _this.setState({
          loading: PROGRESS_ERROR
        });
      }
    })();
  }

  fetchPackages(query) {
    return _asyncToGenerator(function* () {
      const res = yield (0, _algolia.search)({
        query,
        attributesToRetrieve: ['name', 'version', 'description', 'owner', 'humanDownloadsLast30Days'],
        offset: 0,
        length: 5
      });
      const {
        hits
      } = res;
      const packages = hits.map(hit => _objectSpread({}, hit, {
        _cell: getCellPadding(hits, hit)
      }));
      return packages;
    })();
  }

  handleTogglePackage(pkg) {
    const {
      selectedPackages,
      loading
    } = this.state;

    if (loading !== PROGRESS_LOADED) {
      return;
    }

    const exists = selectedPackages.some(({
      objectID
    }) => objectID === pkg.objectID);

    if (exists) {
      this.setState({
        query: '',
        selectedPackages: selectedPackages.filter(({
          objectID
        }) => objectID !== pkg.objectID)
      });
    } else {
      this.setState({
        query: '',
        selectedPackages: [...selectedPackages, pkg]
      });
    }
  }

  handleInstall() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      const {
        query,
        selectedPackages
      } = _this2.state;

      if (notEmpty(query)) {
        return;
      }

      if (isEmpty(selectedPackages)) {
        _this2.props.onExit();
      } // ENV


      const isDev = _this2.props.dev;
      const yarn = yield shouldUseYarn();
      const env = yarn ? 'yarnpkg' : 'npm';
      const arg = yarn ? ['add'] : ['install', '--save'];
      const devArg = yarn ? '-D' : '--save-dev'; // Packages

      const packages = selectedPackages.map(pkg => pkg.name);
      const args = [...arg, ...packages, ...(isDev ? [devArg] : [])]; // Install the queries

      try {
        yield _execa.default.sync(env, args, {
          stdio: `inherit`
        });
      } catch (err) {
        throw err;
      }

      _this2.props.onExit();
    })();
  }

}

var _default = Emma;
exports.default = _default;